package COM.tolstoy.jconfig;

import java.awt.*;
import java.awt.image.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.Date;

/**
Used by the DiskBrowser class. Displays information about a file, alias, directory, or volume.
The disk object being displayed is set and retrieved by the BrowserFrame which contains this
panel.


Feel free to modify this file as you wish.



@author JConfig Copyright (c) 1997-98 Samizdat Productions. All Rights Reserved.

*/

public class BrowserPanel extends Panel {
	DiskObject				diskObject;
	int						smIconData[], smIconW, smIconH,
								lgIconData[], lgIconW, lgIconH,
								which;

	public BrowserPanel( int i ) {
		diskObject = null;
		smIconData = null;
		smIconW = smIconH = 0;
		lgIconData = null;
		lgIconW = lgIconH = 0;
		which = i;
	}

/*
Depending on which type of disk object we hold, display its information. Note that since DiskAlias
extends DiskFile, we check for it first.
*/

	void showDiskInformation( Graphics g ) {
		showLargeIcon( g, 10, 10 );
		showSmallIcon( g, 50, 10 );
		
		if ( diskObject instanceof DiskAlias )
			showDiskAliasStrings( g, 100, 10 );
		else if ( diskObject instanceof DiskFile )
			showDiskFileStrings( g, 100, 10 );
		else if ( diskObject instanceof DiskVolume )
			showDiskVolumeStrings( g, 100, 10 );
		else
			System.out.println( "diskObject of unknown class: " + diskObject );
	}

/*
Get the icon bundle for the disk object, and display the large icon.
*/

	void showLargeIcon( Graphics g, int x, int y ) {
		MemoryImageSource		mis;
		Image						image;
		IconBundle				ib;
		int						theErr;

		ib = diskObject.getIconBundle();
		if ( ib == null ) {
			System.out.println( "can't get icon bundle from " + diskObject.getName() );
			return;
		}

		if ( lgIconData == null ||
		lgIconW != ib.getIconWidth( IconBundle.kLargeIcon ) ||
		lgIconH != ib.getIconHeight( IconBundle.kLargeIcon ) ) {
			lgIconW = ib.getIconWidth( IconBundle.kLargeIcon );
			lgIconH = ib.getIconHeight( IconBundle.kLargeIcon );
			lgIconData = new int[ lgIconW * lgIconH ];
		}

		theErr = ib.getIcon( IconBundle.kLargeIcon, IconBundle.kXformNone, IconBundle.kAlignNone, lgIconData );

		if ( theErr != ErrCodes.kNoErr ) {
			System.out.println( "got err=" + theErr + " from icon bundle for " + diskObject.getName() );
			return;
		}

		mis = new MemoryImageSource( lgIconW, lgIconH, lgIconData, 0, lgIconW );

		image = createImage( mis );

		g.drawImage( image, x, y, null );
	}

/*
Similar to the preceding method.
*/

	void showSmallIcon( Graphics g, int x, int y ) {
		MemoryImageSource		mis;
		Image						image;
		IconBundle				ib;
		int						theErr;

		ib = diskObject.getIconBundle();
		if ( ib == null ) {
			System.out.println( "can't get icon bundle from " + diskObject.getName() );
			return;
		}

		if ( smIconData == null ||
		smIconW != ib.getIconWidth( IconBundle.kSmallIcon ) ||
		smIconH != ib.getIconHeight( IconBundle.kSmallIcon ) ) {
			smIconW = ib.getIconWidth( IconBundle.kSmallIcon );
			smIconH = ib.getIconHeight( IconBundle.kSmallIcon );
			smIconData = new int[ smIconW * smIconH ];
		}

		theErr = ib.getIcon( IconBundle.kSmallIcon, IconBundle.kXformNone, IconBundle.kAlignNone, smIconData );

		if ( theErr != ErrCodes.kNoErr ) {
			System.out.println( "got err=" + theErr + " from icon bundle for " + diskObject.getName() );
			return;
		}

		mis = new MemoryImageSource( smIconW, smIconH, smIconData, 0, smIconW );

		image = createImage( mis );

		g.drawImage( image, x, y, null );
	}

/*
Show information for a disk file.
*/

	void showDiskFileStrings( Graphics g, int x, int y ) {
		VersionInfo			versionInfo;
		DiskFile			df;
		File				jFile;
		String				s1, s2, s3, s4;
		int					theErr, finfo[], flags;
		
		df = (DiskFile) diskObject;

		finfo = new int[ DiskFile.kFInfoLen ];

		theErr = df.getFinderInfo( finfo );
		if ( theErr != ErrCodes.kNoErr && theErr != ErrCodes.kUnimplementedErr )
			System.out.println( "got err=" + theErr + " from getFinderInfo for " + df.getName() );
		flags = df.getFlags();
		versionInfo = df.getVersion();
		jFile = df.getFile();

		if ( ( flags & DiskFile.kIsDirectory ) != 0 )
			s1 = "FOLDER ";
		else
			s1 = "FILE ";

		s1 += "@" + Integer.toHexString( df.hashCode() ) + ": " +  df.getName();

		s2 = "flags=" + df.diskFileFlagsToString( flags ) +
				" ( 0x" + Integer.toHexString( flags ) + " )" +
				", creationDate=" + df.getCreationDate();
	
		if ( versionInfo == null )
			s2 += ", no version info";
		else
			s2 += ", version string: " + versionInfo.getVersionString();

		if ( jFile == null )
			s3 = "java.io.File is null";
		else
			s3 = "java.io.File=" + jFile.getPath();

		if ( !jFile.exists() )
			s3 += "** java.io.File.exists() == false **";


		s4 = "creator=" + JUtils.intToAscii( finfo[ DiskFile.kFInfoCreatorOffset ] ) +
				", type=" + JUtils.intToAscii( finfo[ DiskFile.kFInfoTypeOffset ] ) +
				", color coding=" + df.getColorCoding() +
				", file size=" + df.getFileSize() +
				", res fork size=" + df.getResourceForkSize();
		
		g.setColor( Color.red );
		g.drawString( s1, x, y );
		g.drawString( s2, x, y + 15 );
		g.drawString( s3, x, y + 30 );
		g.drawString( s4, x, y + 45 );
	}

/*
Show information for an alias.
*/

	void showDiskAliasStrings( Graphics g, int x, int y ) {
		DiskObject		resolvedObject;
		DiskAlias		da;
		String			s1, s2;
		int				type;

		da = (DiskAlias) diskObject;

		s1 = "ALIAS @" + Integer.toHexString( da.hashCode() ) + ": " +  da.getName();

		type = da.getAliasType();
		if ( type == DiskAlias.kToVolume ) s2 = "to volume";
		else if ( type == DiskAlias.kToDirectory ) s2 = "to folder";
		else if ( type == DiskAlias.kToFile ) s2 = "to file";
		else if ( type == DiskAlias.kToOther ) s2 = "to unknown";
		else if ( type == DiskAlias.kNoLongerAlias ) s2 = "no longer alias";
		else s2 = "unknown type value";

		try {
			resolvedObject = FileRegistry.resolveAlias( da, FileRegistry.kResolveAliasNoUI );
			if ( resolvedObject == null )
				s2 += ", resolveAlias returned null";
			else
				s2 += ": to " + resolvedObject.getName();
		}
		catch ( Exception e ) {
			s2 += " got exception resolving: " + e;
		}

		g.setColor( Color.red );
		g.drawString( s1, x, y );
		g.drawString( s2, x, y + 15 );
	}

/*
Show information for a volume.
*/

	void showDiskVolumeStrings( Graphics g, int x, int y ) {
		DiskVolume			dv;
		File				jFile;
		String				s1, s2, s3, s4;
		
		dv = (DiskVolume) diskObject;
		
		jFile = dv.getFile();

		s1 = "VOLUME @" + Integer.toHexString( dv.hashCode() ) + ": " +  dv.getName();

		s2 = "prefix=" + dv.getPrefix() +
				", color coding=" + dv.getColorCoding() +
				", ref num=" + dv.getReferenceNumber() +
				", creationDate=" + dv.getCreationDate();
	
		if ( jFile == null )
			s3 = "java.io.File is null";
		else
			s3 = "java.io.File=" + jFile.getPath();

		if ( !jFile.exists() )
			s3 += "** java.io.File.exists() == false **"; 

		s4 = "max capacity=" + dv.getMaxCapacity() +
				", free space=" + dv.getFreeSpace();

		g.setColor( Color.red );
		g.drawString( s1, x, y );
		g.drawString( s2, x, y + 15 );
		g.drawString( s3, x, y + 30 );
		g.drawString( s4, x, y + 45 );
	}

	public DiskObject getDiskObject() {
		return diskObject;
	}
	
	public void setDiskObject( DiskObject dob ) {
		diskObject = dob;
	}

	public Dimension preferredSize() {
		return new Dimension( 500, 60 );
	}

	public void paint( Graphics g ) {
		Dimension		dims;

		dims = size();
		if ( diskObject == null ) {
			g.setColor( Color.darkGray );
			g.fillRect( 0, 0, dims.width, dims.height );
		}
		else {
			g.setColor( Color.lightGray );
			g.fillRect( 0, 0, dims.width, dims.height );
			g.draw3DRect( 0, 0, dims.width - 1, dims.height - 1, false );
			showDiskInformation( g );
		}
	}
}

