/*
	Tester.java is the source to the test program. The compiled version of this file
	is included in JConfig.zip.

	Copyright  1997 Samizdat Productions. All Rights Reserved.
*/

package COM.tolstoy.jconfig;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

/**
This class is used to test this package on Windows.
*/

public class TesterW {
	private static final String copyrightString = "JConfig Copyright (c) 1997 Samizdat Productions. All Rights Reserved.";
	public static final int		kMySignature = JUtils.asciiToInt( "fred" );

	static PrintStream			outStream = null;

	public static void main( String args[] ) {
		FinderInfo			finfo;
		FileExtension		txtExtension;
		AppFile				simpleAppFile = null;
		AppProcess			simpleAppProcess = null;
		File				curDir;
		
		try {
			curDir = new File( System.getProperty( "user.dir" ) );

			outStream = new PrintStream( new FileOutputStream( new File( curDir, "results.txt" ) ) );

			FileRegistry.initialize( curDir, kMySignature );
			if ( !FileRegistry.isInited() ) {
				System.out.println( "Please check your configuration." );
				return;
			}

			finfo = new FinderInfo( JUtils.asciiToInt( "ttxt" ), JUtils.asciiToInt( "TEXT" ) );
			txtExtension = new FileExtension( ".txt" );

			tryFindAppByExtension( txtExtension );
			
			simpleAppFile = tryFindAppByFinderInfo( finfo );

			if ( simpleAppFile != null ) {
				tryGetFileTypes( simpleAppFile );
				simpleAppProcess = tryOpenApp( simpleAppFile );
				tryDisplayApplicationIcon( simpleAppFile, IconBundle.kLargeIcon );
			}
			
			tryLaunchingURL( "http://www.tolstoy.com" );

			tryIteratingOnConfigEntries();

			tryFindFinderInfo( txtExtension );

			tryFindExtensions( finfo );

			tryFindAppsByName( "NOTEPAD" );

			if ( simpleAppProcess != null ) {
				tryMaximizeApp( simpleAppProcess );
				tryQuitProcess( simpleAppProcess );
			}
		}
		catch ( Exception e ) {
			System.out.println( "problems: " + e );
			e.printStackTrace( System.out );
		}

		outStream.println( "" );
		outStream.println( "*** Ciao." );
		outStream.close();
	}

/*
Try to launch the user's browser. 'url' must be a full url, i.e., complete with 'http', or 'mailto',
etc. Presently, this method will fail if the user has not used Internet Config to set their
default browser.
*/

	private static int tryLaunchingURL( String url ) {
		int			theErr;

		outStream.println( "" );
		outStream.println( "*** Launching a URL..." );

		theErr = FileRegistry.launchURL( url, 0, null );
		
		if ( theErr != ErrCodes.kNoErr )
			outStream.println( "can't launch browser, theErr=" + theErr );
		
		return theErr;
	}

/*
Using the FileRegistry, try to find an application corresponding to the given FinderInfo.
*/

	private static AppFile tryFindAppByFinderInfo( FinderInfo finfo ) {
		AppFile				theApps[];

		outStream.println( "" );
		outStream.println( "*** Calling FileRegistry.getApps for " + finfo );

		theApps = FileRegistry.getApps( finfo, 1, 0 );
		if ( theApps == null || theApps[ 0 ] == null ) {
			outStream.println( "can't find apps using " + finfo );
			return null;
		}

		theApps[ 0 ].dumpInfo( outStream, "  " );
		
		return theApps[ 0 ];
	}

/*
Using the FileRegistry, try to find an application corresponding to the given FileExtension.
*/

	private static AppFile tryFindAppByExtension( FileExtension ext ) {
		AppFile				theApps[];

		outStream.println( "" );
		outStream.println( "*** Calling FileRegistry.getApps for " + ext );

		theApps = FileRegistry.getApps( ext, 1, 0 );
		if ( theApps == null || theApps[ 0 ] == null ) {
			outStream.println( "can't find apps using " + ext );
			return null;
		}

		theApps[ 0 ].dumpInfo( outStream, "  " );
		
		return theApps[ 0 ];
	}

/*
Try to print out the file types associated with the given application.
*/

	private static void tryGetFileTypes( AppFile appFile ) {
		FileType			fileTypes[];
		int					i;
	 	
		outStream.println( "" );
		outStream.println( "*** Getting the FileTypes of an application..." );

		fileTypes = appFile.getFileTypes( 10 );
		if ( fileTypes == null ) {
			outStream.println( "application has no file types" );
			return;
		}

		outStream.println( "application has " + fileTypes.length + " file types:" );

		for ( i = 0; i < fileTypes.length; i++ )
			outStream.println( "  " + fileTypes[ i ] );
	}

/*
Try to create an instance of the given application.

After using a FileDialog to allow the user to open a file, the full path name
of that file is added as an argument to the open doc command, and then the
performCommand() method of the AppFile is called to execute the command.

Returns the resulting AppProcess, or null if an error occurs.
*/

	private static AppProcess tryOpenApp( AppFile appFile ) {
		AppCommand			theCommand;
		AppProcess			theProcess;
		String				fileName;

		outStream.println( "" );
		outStream.println( "*** Calling performCommand to open an app..." );

		theCommand = appFile.getCommand( AppCommand.kAppCommandOpenDoc );
		if ( theCommand == null ) {
			outStream.println( "can't get kAppCommandOpenDoc from process" );
			return null;
		}

		outStream.println( "theCommand = " + theCommand );

		fileName = JUtils.getFileNameFromUser();
		if ( fileName == null ) {
			outStream.println( "-- cancelled --" );
			return null;
		}

		theCommand.addArg( fileName );

		theProcess = appFile.performCommand( theCommand, AppFile.kDontSwitchLayer );
		if ( theProcess == null )
			outStream.println( "couldn't create process" );
		else {
			theProcess.dumpInfo( outStream, "" );
			outStream.println( "The AppFile now should have one spawn:" );
			appFile.dumpInfo( outStream, "  " );
		}

		return theProcess;
	}

/*
Quit the given process.
*/

	private static void tryQuitProcess( AppProcess appProcess ) {
		AppCommand		theCommand;
		int				theErr;

		outStream.println( "" );
		outStream.println( "*** Calling performCommand to quit a process..." );

		if ( !appProcess.isRunning() ) {
			System.out.println( "process is not running" );
			return;
		}

		theCommand = appProcess.getCommand( AppCommand.kAppCommandQuit );
		if ( theCommand == null ) {
			outStream.println( "can't get kAppCommandQuit from process" );
			return;
		}

		outStream.println( "theCommand = " + theCommand );

		theErr = appProcess.performCommand( theCommand, 0 );
		outStream.println( "theErr=" + theErr );
	}

/*
Display every 20th entry in the file mappings database.
*/

	private static void tryIteratingOnConfigEntries() {
		outStream.println( "" );
		outStream.println( "*** Iterating on ConfigEntry's..." );
		
		FileRegistry.iterate( new BasicCEVisitor( outStream ) );
	}

/*
Try to find FinderInfo objects corresponding to the given FileExtension.
*/

	private static void tryFindFinderInfo( FileExtension ext ) {
		FinderInfo		infoArray[];
		int				i, numWritten;

		outStream.println( "" );
		outStream.println( "*** Finding FinderInfo's associated with extension: " + ext );

		infoArray = FileRegistry.findFinderInfo( ext, 10 );
		if ( infoArray == null ) {
			outStream.println( "can't find finder info for " + ext );
			return;
		}

		for ( i = 0, numWritten = 0; i < infoArray.length; i++ ) {
			if ( infoArray[ i ] != null ) {
				outStream.println( "at " + i + " is " + infoArray[ i ] );
				++numWritten;
			}
		}
		
		if ( numWritten == 0 )
			outStream.println( "can't find finder info for " + ext );
	}

/*
Try to find FileExtension objects corresponding to the given FinderInfo.
*/

	private static void tryFindExtensions( FinderInfo finfo ) {
		FileExtension		extArray[];
		int					i, numWritten;

		outStream.println( "" );
		outStream.println( "*** Finding extensions associated with: " + finfo );

		extArray = FileRegistry.findExtensions( finfo, 10 );
		if ( extArray == null ) {
			outStream.println( "can't find extensions for " + finfo );
			return;
		}

		for ( i = 0, numWritten = 0; i < extArray.length; i++ ) {
			if ( extArray[ i ] != null ) {
				outStream.println( "at " + i + " is " + extArray[ i ] );
				++numWritten;
			}
		}
		
		if ( numWritten == 0 )
			outStream.println( "can't find extensions for " + finfo );
	}

/*
Try to find the applications whose name contains the given string.
*/

	private static void tryFindAppsByName( String name ) {
		AppFile			theApps[];
		int				i, numWritten;

		outStream.println( "" );
		outStream.println( "*** Finding apps whose name is " + name );

		theApps = FileRegistry.getApps( name, 10, 0 );
		if ( theApps == null ) {
			outStream.println( "can't find apps containing *" + name + "*" );
			return;
		}

		for ( i = 0, numWritten = 0; i < theApps.length; i++ ) {
			if ( theApps[ i ] != null ) {
				theApps[ i ].dumpInfo( outStream, "  " );
				++numWritten;
			}
		}
		
		if ( numWritten == 0 )
			outStream.println( "can't find apps containing *" + name + "*" );
	}

/*
Try to minimize the given process.
*/

	private static void tryMaximizeApp( AppProcess appProcess ) {
		int			theErr;
		
		outStream.println( "" );
		outStream.println( "*** Maximizing application..." );

		if ( !appProcess.isRunning() ) {
			System.out.println( "process is not running" );
			return;
		}

		theErr = appProcess.move( null, AppProcess.kMaximize, 0 );
		
		if ( theErr != ErrCodes.kNoErr )
			outStream.println( "theErr=" + theErr );
	}

/*
Try to display an application icon for the given file.
*/

	private static void tryDisplayApplicationIcon( AppFile appFile, int whichIcon ) {
		IconBundle			iconBundle;
		IconDisplayPanel	idp;
		FileType				fileType;
		int					bits[], numBits, theErr;
		IconDisplayFrame	aFrame;

		outStream.println( "" );
		outStream.println( "*** Displaying icon for application..." );

		fileType = new FileType( new FileExtension( ".exe" ) );
//		fileType = new FileType( new FinderInfo( 0, JUtils.asciiToInt( "APPL" ) ) );

		iconBundle = appFile.getIconBundle( fileType );
		if ( iconBundle == null ) {
			outStream.println( "app has no IconBundle" );
			return;
		}

		numBits = iconBundle.getIconWidth( whichIcon ) * iconBundle.getIconHeight( whichIcon );
		bits = new int[ numBits ];

		theErr = iconBundle.getIcon( whichIcon, 0, 0, bits );
		if ( theErr != ErrCodes.kNoErr ) {
			outStream.println( "iconBundle.getIcon returned null" );
			return;
		}

		idp = new IconDisplayPanel( bits, iconBundle.getIconWidth( whichIcon ),
											iconBundle.getIconHeight( whichIcon ) );

		aFrame = new IconDisplayFrame( "Icon Test" );
		aFrame.setLayout( new BorderLayout( 0, 0 ) );
		aFrame.resize( 100, 100 );
		aFrame.add( "Center", idp );
		aFrame.show();
	}
}

